from vpython import *
#GlowScript 3.0 VPython
## three dimensional simulation of the three-body problem
## using the Python Visual module for the animation
## see http://www.vpython.org/
## this version is hacked to show Kepler's second law

global threebody
scene = display(title="University of Glasgow A1 Dynamical Astronomy. Kepler's Second Law (press 1-2). RH mouse rotates, LH+RH zooms.", width=1280, height=800)

## set up the masses. m1 is missing in 2-body systems
m1 = sphere(color=color.red,   make_trail=True, interval=2, retain=1000)
m2 = sphere(color=color.white,make_trail=True, interval=2, retain=1000, texture="https://mw1.google.com/mw-earth-vectordb/kml-samples/bmng12/files/BMNG-Mar.jpg")
m3 = sphere(color=color.cyan, make_trail=True, interval=2, retain=1000)
kepler = curve()
scene.lights=[]
decsun = 20/180*pi
lamp = distant_light(direction = vector(cos(decsun)/sqrt(2),-cos(decsun)/sqrt(2),sin(decsun)),color=color.white)# the Sun221!

## move to the centre of mass frame and adjust sizes for constant density
def resetscene(m1,m2,m3):
    vcentre=(m1.mass*m1.vel+m2.mass*m2.vel+m3.mass*m3.vel)/(m1.mass+m2.mass+m3.mass)
    rcentre=(m1.mass*m1.pos+m2.mass*m2.pos+m3.mass*m3.pos)/(m1.mass+m2.mass+m3.mass)
    for a in [m1, m2, m3]:
        a.vel -= vcentre
        a.radius = 0.5*a.mass**(1.0/3.0)
        a.clear_trail() 
    kepler.clear()

def dydt(y):
    deriv = [vector(0,0,0)] * 6
    r12 = y[0]-y[2] 
    r23 = y[2]-y[4] 
    r31 = y[4]-y[0]
    r12c = r12.hat/mag2(r12)
    r23c = r23.hat/mag2(r23)
    r31c = r31.hat/mag2(r31)
    deriv[1] = -m2.mass*r12c + m3.mass*r31c
    deriv[3] = -m3.mass*r23c + m1.mass*r12c
    deriv[5] = -m1.mass*r31c + m2.mass*r23c
    deriv[0] = y[1];  deriv[2] = y[3];  deriv[4] = y[5]
    return mult(dt, deriv)
    
def mult(f,v):
    c = [vec(0,0,0)] * 6
    for i in range(0,6):
     c[i] = f*v[i]
    return c
    
def add(j,k):
    c= [vec(0,0,0)] * 6
    for i in range(0,6):
     c[i] = j[i] + k[i]
    return c


def keyInput(evt):
    global threebody,dt
    s = evt.key
    if (s == '1'): ##circular
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( 0,0,0);   m2.vel=vector( 0, 0,0); m2.mass=10
        m3.pos=vector( 10,0,0);  m3.vel=vector(0,1,0);  m3.mass=0.01
        scene.range=20
        resetscene(m1,m2,m3)
        threebody = False;
    if (s == '2'):
        m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
        m2.pos=vector( 5,0,0);   m2.vel=vector(0,0,0);   m2.mass=10.0
        m3.pos=vector(-10,0,0);  m3.vel=vector(0,-0.5,0); m3.mass=0.01
        scene.range=15
        resetscene(m1,m2,m3)
        threebody= False


scene.bind('keydown', keyInput)

## initiate with the simplest system
m1.pos=vector( 100,0,0); m1.vel=vector(0,0,0);   m1.mass=0
m2.pos=vector( 0,0,0);   m2.vel=vector( 0, 0,0); m2.mass=10
m3.pos=vector( 10,0,0);  m3.vel=vector(0,1,0);  m3.mass=0.01
scene.range=20
resetscene(m1,m2,m3)
m2.rotate(angle = pi/2, axis=vector(1,0,0),origin=m2.pos)
threebody = False;

dt = 0.1 ## the time step.  Keep this as small as possible.
c = 0    
    
while True:
    rate(100)
    y = [ m1.pos, m1.vel, m2.pos, m2.vel, m3.pos, m3.vel] 
    k1 = dydt(y)
    k2 = dydt(add(y, mult( 0.5, k1)))
    k3 = dydt(add(y, mult( 0.5, k2))) 
    k4 = dydt(add(y,k3))
    dy = add(add( mult( (1.0/6.0), k1), mult( (1.0/3.0), k2)),add( mult( (1.0/3.0), k3) , mult( (1.0/6.0), k4)))
    if threebody:    
        m1.pos += dy[0]; m1.vel += dy[1]
    m2.pos += dy[2]; m2.vel += dy[3]
    m3.pos += dy[4]; m3.vel += dy[5]
    m2.rotate(angle = 0.0003,axis=vector(0,0,1),origin=m2.pos)
    c+=1
    if (c % 30) == 0:
      kepler.append((m1.mass*m1.pos+m2.mass*m2.pos+m3.mass*m3.pos)/(m1.mass+m2.mass+m3.mass))
      kepler.append(pos=m3.pos,retain=12)










